﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Reflection;
using System.Windows.Controls;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Data;

namespace TicketID_231616_CaseInsensitiveFiltering
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            this.InitializeComponent();

            this.playersGrid.ItemsSource = Club.GetPlayers();
            this.playersGrid.Filtering += this.OnFiltering;
        }

        private void OnFiltering(object sender, GridViewFilteringEventArgs e)
        {
            var newFilters = new List<IFilterDescriptor>();
            foreach (FilterDescriptor filterDescriptor in e.AddedDescriptors)
            {
                var newFilter = new CaseInsensitiveFilterDescriptor();
                newFilter.Member = filterDescriptor.Member;
                newFilter.Operator = filterDescriptor.Operator;
                newFilter.Value = filterDescriptor.Value;
                newFilters.Add(newFilter);
                break;
            }

            if (newFilters.Count > 0)
            {
                e.AddedDescriptors.Clear();
                this.playersGrid.FilterDescriptors.Add(newFilters[0]);
            }

            if (e.RemovedDescriptors.Count > 0)
            {
                FilterDescriptor toRemove = null;
                foreach (IFilterDescriptor fd in this.playersGrid.FilterDescriptors)
                {
                    if (fd is FilterDescriptor)
                    {
                        if (((FilterDescriptor) fd).Member == e.RemovedDescriptors[0].Member)
                        {
                            toRemove = (FilterDescriptor) fd;
                            break;
                        }
                    }
                }

                this.playersGrid.FilterDescriptors.Remove(toRemove);
            }
        }
    }

    public class CaseInsensitiveFilterDescriptor : FilterDescriptor
    {
        protected override Expression CreateFilterExpression(ParameterExpression instance)
        {
            if (base.Operator == FilterOperator.IsEqualTo)
            {
                MethodInfo toLowerMethod = typeof (string).GetMethod("ToLower", new Type[0]);
                MemberExpression member = Expression.Property(instance, base.Member);
                MethodCallExpression toLower = Expression.Call(member, toLowerMethod);
                LambdaExpression val = Expression.Lambda(toLower, instance);
                return Expression.Equal(toLower, Expression.Constant(base.Value.ToString().ToLower()));
            }

            return base.CreateFilterExpression(instance);
        }
    }
}